//*Mancave Effects K40 Laser Cooling Guard
//*This code watches the flow rate and temperature of your K40 Laser cooling liquid. 
//*If the flow stops, the relay will automaticly cut the output of the laser tube.
//*It displays the temperature in deg C or deg. F and the flow rate in L/min.
//*The display updates the values every second.
//*Changeing deg. C to deg. F is as easy as swapping "lcd.print(Tc);" to "lcd.print(Tf);" in the code below.
//*There is a detailed video on my Youtube Channel Mancave Effects about how to hook everything up.


#include<math.h>                                      //math for converting the thermistor data
#include "HardwareSerial.h"
#include "Wire.h"
#include "Adafruit_LiquidCrystal.h"                   //This Library can simply be downloaded in Arduino (Tools/Manage Libraries)

Adafruit_LiquidCrystal lcd(12, 11, 3, 4, 5, 6);

int Relais = 0;
int flowPin = 2;
int ThermistorPin = A0;
int Vo;

double flowRate;                                      // This is the value after the math that can be displayed or worked with 
volatile int count;
float R1 = 10000;
float logR2, R2, T, Tc, Tf;
float c1 = 1.009249522e-03, c2 = 2.378405444e-04, c3 = 2.019202697e-07;


void setup()
{
  Serial.end();
  lcd.begin(16, 2);
  lcd.setCursor(1, 0);
  lcd.print("MCE K40 Laser");                         // You can change this start up text to whatever you want, it appears on the display when you turn on the system
  lcd.setCursor(1, 1);
  lcd.print("Cooling Guard");                         // You can change this start up text to whatever you want, it appears on the display when you turn on the system
  delay(3000);
    for (int scrollCounter = 0; scrollCounter < 16; scrollCounter++)
  {
   lcd.scrollDisplayLeft();
   delay(50);
  }
 
  pinMode(flowPin, INPUT);         
  attachInterrupt(0, Flow, RISING); 
  pinMode(0, OUTPUT);
  }

void loop()
{
  Vo = analogRead(ThermistorPin);
  R2 = R1 * (1023.0 / (float)Vo - 1.0);
  logR2 = log(R2);
  T = (1.0 / (c1 + c2*logR2 + c3*logR2*logR2*logR2));
  Tc = T - 273.15;
  Tf = (Tc * 9.0)/ 5.0 + 32.0; 
  
  count = 0;      
  interrupts();  
  delay(1000);   
  noInterrupts(); 
  flowRate = (count / 2.25);                          // tweak the Flow Sensor depending on minimum Flow
 
  lcd.begin(16, 2);

  if (flowRate >= 0.5){                               // turns off the relay when "flowRate" is under 0.5L/min and opens the laser switch
     digitalWrite(Relais, LOW);                       
  
  lcd.print("Temp:");
  lcd.setCursor(6, 0);
  lcd.print(Tc);                                      // Prints the temperature of the cooling liquid on the display. To change this to Fahrenheit swap "Tc" with "Tf"
  lcd.setCursor(11, 0);
  lcd.print((char)223);
  lcd.setCursor(12, 0);
  lcd.print("C");                                     // When changing to Fahrenheit swap the "C" to "F"

  lcd.setCursor(0, 1);
  lcd.print("Flow:");
  lcd.setCursor(6, 1);
  lcd.print(flowRate);
  lcd.setCursor(11, 1);
  lcd.print("L/Min");
  }
  else {
     digitalWrite(Relais, HIGH);
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("PUMP ERROR");
  delay(1000);
  }}

void Flow()
{
   count++; 
}
